/*
 *
 * Sada funkci pro obsluhu terminalu
 *
 */

#include <stdint.h>
#include "io.h"
#include "term.h"

#define TERM_W	(80)  /* sirka terminalu */
#define TERM_H	(25)  /* vyska terminalu */

struct term {
	unsigned int row;
	unsigned int column;
	unsigned char color;
} term;


static inline void term_relocate_cursor(unsigned int row, unsigned column);

/**
 * nastavi vlastnosti terminalu na vhodne hodnoty
 */
void term_init()
{
	term.row    = 0;
	term.column = 0;
	term.color  = 0x07;
	term_relocate_cursor(0, 0);
}

/**
 * Vypise na terminal prave jeden znak
 */
void term_putc(char c)
{
	unsigned char *vram = (unsigned char *) 0xb8000;
	if (c < 0x20) { /* zpracovani ridicich znaku */
		if (c == '\n') term.row++;
		if (c == '\r') term.column = 0;
	} else {        /* zpracovani beznych znaku */ 
		unsigned int index = ((term.row * TERM_W) + term.column) * 2;
		vram[index]     = c;
		vram[index + 1] = term.color;
		term.column++;
	}

	if (term.column >= TERM_W) { /* preteceni radku */
		term.column = 0;
		term.row++;
	}

	if (term.row >= TERM_H) {    /* preteceni obrazovky => scroll */ 
		for (unsigned int i = TERM_W; i < TERM_W * TERM_H; i++) {
			vram[(i - TERM_W) * 2]     = vram[i * 2];
			vram[(i - TERM_W) * 2 + 1] = vram[i * 2 + 1];
		}
		unsigned char *last_line = vram + (TERM_W * (TERM_H - 1)) * 2;
		for (unsigned i = 0; i < TERM_W; i++) {
			last_line[i * 2]     = ' ';
			last_line[i * 2 + 1] = term.color;
		}
		term.row = TERM_H - 1;
	}
	term_relocate_cursor(term.row, term.column);
}

/**
 * Zmeni barvy znaku vypisovanych na terminal
 */
void term_color(int fg, int bg)
{
	term.color = ((bg << 4) & 0xf0) | (fg & 0x0f);
}

/**
 * Presune kurzor na dane souradnice
 */
static inline void term_relocate_cursor(unsigned int row, unsigned column)
{	
        uint16_t pos = row * TERM_W + column;       // pozice kurzoru
        outb(0x3d4, 0x0f);                          // prikaz pro zapis LSB pozice
        outb(0x3d5, (uint8_t) (pos & 0xff));        // hodnota LSB pozice
        outb(0x3d4, 0x0e);                          // prikaz pro zapis MSB pozice
        outb(0x3d5, (uint8_t) ((pos >> 8) & 0xff)); // hodnota MSB
}

/**
 * Vraci scan code prave stisknute nebo uvolnene klavesy.
 * Blokujici operace.
 */
unsigned char keyb_scancode()
{
	while (1) {
		uint8_t status = inb(0x64);
		if (status & 0x01) {
			uint8_t scan_code = inb(0x60);
			return scan_code;
		}
	}
}


